package cz.muni.fi.pb162.collections;


/**
 * Rozhrani reprezentujici vytisk knihy v knihovne. Kazda kniha je 
 * identifikovana unikatnim ID typu int. Kazda kniha si pamatuje autora a nazev 
 * v podobe dvou textovych retezcu. Kniha muze byt pujcena nebo volna. Kniha si 
 * pamatuje vsechny ctenare, kteri si ji vypujcili. Kniha si pamatuje odkaz na 
 * registr ctenaru.
 * 
 * @author Radek Oslejsek
 * @version 2007-11-15
 */

public interface Book
{
    /**
     * Vraci nazev knihy.
     * 
     * @return nazev knihy
     */
    String getTitle();
    
    /**
     * Vraci autora knihy.
     * 
     * @return autora knihy
     */
    String getAuthor();
    
    /**
     * Vraci identifikator knihy.
     *
     * @return identifikator knihy
     */
    int getId();
    
    /**
     *  Vypujceni knihy. Pokud kniha neni pujcena, a ctenar je registrovan
     *  v registru ctenaru, nastavi knihu na pujcenou a zaradi ctenare do
     *  vypujcek knihy, viz countLoans()
     * 
     * @param reader ctenar zadajici o vypujceni
     * @return false pokud je jiz kniha pujcena, nebo pokud ctenar neni v registru
     */
    boolean borrow(Reader reader);
    
    /**
     * Vraceni knihy.
     * 
     * @return true pokud je kniha skutecne pujcena
     */
    boolean giveBack();

    /**
     * Zjisti pocet vypujcek dane knihy ctenarem.
     * 
     * @param reader kontrolovany ctenar
     * @return pocet vypujcek knihy ctenarem
     */
    int countLoans(Reader reader);
    
    /**
     * <p>Indikuje, zdali je nejaky objekt stejny jako tato kniha.
     * Jiny objekt je stejny (ekvivalentni) jako tato kniha prave
     * tehdy, pokud jde take o knihu a ta ma stejny identifikator 
     * id.</p>
     * 
     * <p>Tato metoda musi samozrejme splnovat obecny kontrakt metody <a 
     * href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Object.html#equals(java.lang.Object)">Object.equals</a>.</p>
     *
     * @param obj Porovnavany objekt
     * @return true, jestli je tento objekt stejny jako argument obj
     */
    boolean equals(Object obj);
    
    
    /**
     * Tato metoda vraci hashovaci kod daneho objektu. Metoda musi splnovat obecny 
     * kontrakt metody <a 
     * href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Object.html#hashCode()">Object.hashCode()</a>.
     * 
     * @return hashovaci k daneho objektu
     */
    int hashCode();       
}
