package cz.muni.fi.pb162.collections;


/**
 * Trida predstavujici ctenare v knihovne. Kazdy ctenar je identifikovany
 * unikatnim ID typu String (napr. rodne cislo) a ma jmeno.
 * 
 * @author Radek Oslejsek
 * @version 2007-11-15
 */

public interface Reader
{
    /**
     * Metoda vraci jmeno ctenare.
     * 
     * @return jmeno ctenare
     */
    String getName();

    /**
     * Metoda vraci identifikator ctenare.
     * 
     * @return indetifikator ctenare
     */
    String getId();
    
    /**
     * <p>Indikuje, zdali je nejaky objekt stejny jako tento ctenar.
     * Jiny objekt je stejny (ekvivalentni) jako tento ctenar prave
     * tehdy, pokud jde take o ctenare a ten ma stejny identifikator 
     * id.</p>
     * 
     * <p>Tato metoda musi samozrejme splnovat obecny kontrakt metody <a 
     * href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Object.html#equals(java.lang.Object)">Object.equals</a>.</p>
     *
     * @param obj Porovnavany objekt
     * @return true, jestli je tento objekt stejny jako argument obj
     */
    boolean equals(Object obj);
    
    
    /**
     * Tato metoda vraci hashovaci kod daneho objektu. Metoda musi splnovat obecny 
     * kontrakt metody <a 
     * href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Object.html#hashCode()">Object.hashCode()</a>.
     * 
     * @return hashovaci k daneho objektu
     */
    int hashCode();       
}
